@@ -40,7 +40,9 @@ let print_deferred_encryption e =
   | None -> Printf.printf "file_encryption_key: None\n"
   | Some _ -> Printf.printf "file_encryption_key: Some\n"
   end;
-  Printf.printf "key = %s\n" (Pdfio.string_of_int_array e.key);
+  Printf.printf "key = ";
+  Array.iter (fun x -> Printf.printf "%C" (char_of_int x)) e.key;
+  Printf.printf "\n";
   Printf.printf "obj = %i, gen = %i\n" e.obj e.gen;
   Printf.printf "keylength = %i\n" e.keylength;
   Printf.printf "r = %i\n" e.r
@@ -878,7 +880,7 @@ let rec nametree_lookup_kids pdf k = function
       | Some result -> Some result
       end
   | Array [] -> None
-  | _ -> raise (PDFError "malformed name tree")
+  | _ -> raise (PDFError "nametree_lookup_kids: malformed name tree")
 
 and array_lookup k = function
   | Array elts ->
@@ -981,4 +983,9 @@ let deep_copy from =
    was_linearized = from.was_linearized;
    saved_encryption = from.saved_encryption}
 
-
+let change_id pdf f =
+  match pdf.trailerdict with
+  | Dictionary d ->
+      pdf.trailerdict <-
+        Dictionary (add "/ID" (generate_id pdf f (fun () -> Random.float 1.)) d)
+  | _ -> raise (PDFError "Bad trailer dictionary")
