@@ -260,7 +260,7 @@ and decrypt_stream
                        Pdf.file_encryption_key = file_encryption_key;
                        Pdf.obj = obj;
                        Pdf.gen = gen;
-                       Pdf.key = key;
+                       Pdf.key = unhashed_key;
                        Pdf.keylength = keylength;
                        Pdf.r = r}
                   in
@@ -563,7 +563,7 @@ let file_encryption_key_aesv3 ?digest iso utf8pw o oe u =
     let d =
       match digest with
       | Some d -> 
-         if Array.length d <> 32 then Printf.printf "file_encryption_key_aesv3 pre-made length %i\n" (Array.length d);
+         if Array.length d <> 32 then Printf.eprintf "file_encryption_key_aesv3 pre-made length %i\n" (Array.length d);
          d
       | None ->
           let i =
@@ -572,7 +572,7 @@ let file_encryption_key_aesv3 ?digest iso utf8pw o oe u =
               Pdfcryptprimitives.sha256 (Pdfio.input_of_string x)))
                 (String.concat "" [utf8pw; String.sub o 40 8; String.sub u 0 48]))
           in
-            if Array.length i <> 32 then Printf.printf "file_encryption_key_aesv3 made length %i\n" (Array.length i);
+            if Array.length i <> 32 then Printf.eprintf "file_encryption_key_aesv3 made length %i\n" (Array.length i);
             i
     in
       Pdfcryptprimitives.aes_decrypt_data ~remove_padding:false 8 d (bytes_of_string (zero_iv ^ oe))
@@ -990,7 +990,7 @@ let encrypt_pdf_AES encrypt_metadata user_pw owner_pw banlist pdf =
       encrypt_pdf_AES_inner owner user p user_pw id encrypt_metadata pdf
 
 let encrypt_pdf_AES256_inner iso encrypt_metadata owner user p perms oe ue id key pdf =
-  if !crypt_debug then Printf.printf "encrypt_pdf_AES256_inner, ISO = %b\n" iso;
+  if !crypt_debug then Printf.eprintf "encrypt_pdf_AES256_inner, ISO = %b\n" iso;
   let crypt_dict =
     Pdf.Dictionary
       ["/Filter", Pdf.Name "/Standard";
@@ -1209,4 +1209,3 @@ let recrypt_pdf ?(renumber=true) pdf pw =
       try recrypt_pdf_user pdf pw with _ -> recrypt_pdf_owner pdf pw
     with
       _ -> raise (Pdf.PDFError "recrypt_pdf failed. Wrong password?")
-
