@@ -1,6 +1,10 @@
 (* General Input and Output *)
 open Pdfutil
 
+(* On 64 bit platforms, bigarray isn't used since max_string is big. If
+ * this is set, we use bigarray always, for testing... *)
+let test_bigarray = false
+
 (* We used to have a type called "bytes" before OCaml did.  In order
    not to break client code using e.g. Pdfio.bytes, we keep the name
    but expose an alias caml_bytes for the built-in type. *)
@@ -36,7 +40,7 @@ let bytes_of_raw b = Long b
 
 (* Make a stream of a given size. *)
 let mkbytes l =
-  if l <= Sys.max_string_length
+  if l <= (if test_bigarray then max_int else Sys.max_string_length)
     then Short (Bytes.create l)
     else Long (Bigarray.Array1.create Bigarray.int8_unsigned Bigarray.c_layout l)
 
@@ -334,7 +338,7 @@ let output_of_bytes s =
     let output_int i =
       if s.pos > bytes_size !(s.data) - 1
         then
-          let newstream = mkbytes (s.pos * 2) in
+          let newstream = mkbytes (max 1 (s.pos * 2)) in
             for x = 0 to bytes_size !(s.data) - 1 do
               bset_unsafe newstream x (bget_unsafe !(s.data) x)
             done;
