@@ -19,3 +19,5 @@ val remove_bookmarks : Pdf.t -> Pdf.t
 (** Add bookmarks to a document, replacing any currently there. *)
 val add_bookmarks : t list -> Pdf.t -> Pdf.t
 
+(** Transform a bookmark's destination *)
+val transform_bookmark : Pdftransform.transform_matrix -> t -> t
