@@ -176,7 +176,12 @@ let rec read_name_tree pdf tree =
 
 let read_name_tree pdf tree =
   let r = read_name_tree pdf tree in
-    List.map (function (Pdf.String s, x) -> (s, x) | _ -> raise (Pdf.PDFError "malformed name tree")) r
+    try
+      map (function (Pdf.String s, x) -> (s, x) | _ -> raise Exit) r
+    with
+      Exit ->
+        Printf.eprintf "Pdfmerge.read_names tree: skipping malformed name tree\n";
+        []
 
 let maxsize = 10 (* Must be at least two *)
 
@@ -188,23 +193,23 @@ let left l = fst (hd l)
 let right l = fst (last l)
 
 let rec build_nt_tree l =
-  if List.length l = 0 then assert false;
-  if List.length l <= maxsize
+  if length l = 0 then assert false;
+  if length l <= maxsize
     then Lf (left l, l, right l)
-    else Br (left l, List.map build_nt_tree (splitinto (List.length l / maxsize) l), right l)
+    else Br (left l, map build_nt_tree (splitinto (length l / maxsize) l), right l)
 
 let rec name_tree_of_nt isroot pdf = function
   Lf (llimit, items, rlimit) ->
     Pdf.Dictionary
-      ([("/Names", Pdf.Array (List.flatten (List.map (fun (k, v) -> [Pdf.String k; v]) items)))] @
+      ([("/Names", Pdf.Array (flatten (map (fun (k, v) -> [Pdf.String k; v]) items)))] @
        if isroot then [] else [("/Limits", Pdf.Array [Pdf.String llimit; Pdf.String rlimit])])
 | Br (llimit, nts, rlimit) ->
     let indirects =
-      let kids = List.map (name_tree_of_nt false pdf) nts in
-        List.map (Pdf.addobj pdf) kids
+      let kids = map (name_tree_of_nt false pdf) nts in
+        map (Pdf.addobj pdf) kids
     in
       Pdf.Dictionary
-       [("/Kids", Pdf.Array (List.map (fun x -> Pdf.Indirect x) indirects));
+       [("/Kids", Pdf.Array (map (fun x -> Pdf.Indirect x) indirects));
         ("/Limits", Pdf.Array [Pdf.String llimit; Pdf.String rlimit])]
 
 let build_name_tree pdf = function
@@ -303,6 +308,67 @@ let catalog_items_from_original_documents pdfs =
   in
     fold_left (fun d (k, v) -> add k v d) [] catalog_entries
 
+(* Optional Content Groups. We merge the /OCProperties entries like this:
+  * a) append /OCGs entries
+  * b) append /Configs entries, or leave absent if everywhere-absent.
+  * c) combine /D dictionary by appending its subentries:
+    *    1) /ON / /OFF arrays, merge or absent if everywhere-absent
+    *    2) /Order ditto
+    *    3) /RBGroups ditto
+    *    4) Use first or any found /Locked /ListMode /AS /Intent /BaseState /Creator /Name *)
+let merge_entries pdf n maindict =
+  flatten
+    (option_map
+       (fun dict -> match Pdf.lookup_direct pdf n dict with Some (Pdf.Array a) -> Some a | _ -> None)
+       maindict)
+
+let merge_default_dictionaries pdf dics =
+  let merge_empty_is_none n =
+    match option_map (Pdf.lookup_direct pdf n) dics with
+    | [] -> []
+    | l -> [(n, Pdf.Array (flatten (option_map (function Pdf.Array a -> Some a | _ -> None) l)))]
+  in
+    let merged_on = merge_empty_is_none "/ON" in
+    let merged_off = merge_empty_is_none "/OFF" in
+    let merged_order = merge_empty_is_none "/Order" in
+    let merged_rbgroups = merge_empty_is_none "/RBGroups" in
+    let simple_copies =
+      let keys =
+        ["/Locked"; "/ListMode"; "/AS"; "/Intent"; "/BaseState"; "/Creator"; "/Name"]
+      in
+      let find_first_item key =
+        match option_map (Pdf.lookup_direct pdf key) dics with
+        | [] -> []
+        | h::_ -> [(key, h)]
+      in
+        flatten (map find_first_item keys)
+    in
+      (*Printf.printf "merged_on length %i\n" (length merged_on);
+      Printf.printf "merged_off length %i\n" (length merged_off);
+      Printf.printf "merged_order length %i\n" (length merged_order);
+      Printf.printf "merged_rbgroups length %i\n" (length merged_rbgroups);
+      Printf.printf "simple_copies length %i\n" (length simple_copies);*)
+      merged_on @ merged_off @ merged_order @ merged_rbgroups @ simple_copies
+
+let merge_optional_content_groups pdf pdfs =
+  let ocp_dicts =
+    option_map
+      (fun pdf -> Pdf.lookup_direct pdf "/OCProperties" (Pdf.catalog_of_pdf pdf))
+      pdfs
+  in
+    if ocp_dicts = [] then None else
+    let merged_ocg_arrays = merge_entries pdf "/OCGs" ocp_dicts in
+    let merged_config_arrays = merge_entries pdf "/Configs" ocp_dicts in
+    let merged_default_dictionary =
+      merge_default_dictionaries pdf (option_map (Pdf.lookup_direct pdf "/D") ocp_dicts)
+    in
+    let new_ocproperties =
+      Pdf.Dictionary
+        ([("/OCGs", Pdf.Array merged_ocg_arrays); ("/D", Pdf.Dictionary merged_default_dictionary)]
+         @ if merged_config_arrays = [] then [] else [("/Configs",  Pdf.Array merged_config_arrays)])
+    in
+      Some (Pdf.addobj pdf new_ocproperties)
+
 let merge_pdfs retain_numbering do_remove_duplicate_fonts names pdfs ranges =
   let pdfs = merge_pdfs_renumber names pdfs in
   let pdfs = merge_pdfs_rename_name_trees names pdfs in
@@ -334,6 +400,12 @@ let merge_pdfs retain_numbering do_remove_duplicate_fonts names pdfs ranges =
                 | Some dests ->
                     add "/Dests" (Pdf.Indirect dests) with_names
             in
+            (* Merge Optional content groups *)
+            let extra_catalog_entries =
+              match merge_optional_content_groups pdf pdfs with
+                None -> extra_catalog_entries
+              | Some ocgpropnum -> add "/OCProperties" (Pdf.Indirect ocgpropnum) extra_catalog_entries
+            in
    let pdf = Pdfpage.add_root pagetree_num extra_catalog_entries pdf in
       (* To sort out annotations etc. *)
       let old_page_numbers =
@@ -354,5 +426,5 @@ let merge_pdfs retain_numbering do_remove_duplicate_fonts names pdfs ranges =
      let pdf = merge_bookmarks changes pdfs ranges pdf in
        Pdfpagelabels.write pdf page_labels;
        if do_remove_duplicate_fonts then remove_duplicate_fonts pdf;
+       Pdf.change_id pdf "";
        pdf
-
