@@ -588,15 +588,161 @@ let add_root pageroot extras pdf =
              Pdf.root = rootnum;
              Pdf.trailerdict = trailerdict'}
 
+(* Make sure to supply refnums to speed it up, if you already have them from a
+ * previous call to Pdf.page_reference_numbers *)
+let pagenumber_of_target ?fastrefnums pdf = function
+ | Pdfdest.NullDestination -> 0
+ | Pdfdest.NamedDestinationElsewhere _ -> 0
+ | Pdfdest.Action _ -> 0
+ | Pdfdest.XYZ (t, _, _, _) | Pdfdest.Fit t | Pdfdest.FitH (t, _) | Pdfdest.FitV (t, _)
+ | Pdfdest.FitR (t, _, _, _, _) | Pdfdest.FitB t | Pdfdest.FitBH (t, _) | Pdfdest.FitBV (t, _) ->
+     match t with
+     | Pdfdest.OtherDocPageNumber i -> i + 1 (* If it's really a Pdfdest.OtherDocPageNumber, you must process this yourself before. *)
+     | Pdfdest.PageObject i ->
+         match fastrefnums with
+         | Some table ->
+             begin try Hashtbl.find table i with Not_found -> 0 end 
+         | None ->
+             match position_1 i (Pdf.page_reference_numbers pdf) with
+             | Some n -> n
+             | None -> 0
+
 (* Return a new PDF containing everything the old one does, but with new pages.
 
 Other objects (e.g destinations in the document outline) may point to the
 individual page objects, so we must renumber these. We can only do this if the
 number of pages are the same. We do this [if change_references is true]. If the
-new and old page lists are of differenct lengths, change_references must be
+new and old page lists are of different lengths, change_references must be
 false, or you must supply the changes (expressed as (from, to) 1-based serial
-number pairs) *)
-let change_pages ?changes change_references basepdf pages' =
+number pairs).
+
+The matrices optional argument, only relevant when change_references is true
+and the number of pages has not changed, gives a list of (page number, matrix)
+pairs which indicate that the page has been transformed. We can then rewrite
+bookmark destinations to reflect the transformed destination positions. We also
+rewrite annotation destinations after the same fashion. *)
+let change_pages_find_matrix dest mattable refnumstable =
+  match dest with
+  | Pdfdest.XYZ (tp, _, _, _) | Pdfdest.FitH (tp, _) | Pdfdest.FitV (tp, _)
+  | Pdfdest.FitR (tp, _, _, _, _) | Pdfdest.FitBH (tp, _) | Pdfdest.FitBV (tp, _) ->
+      begin match tp with
+        Pdfdest.PageObject i ->
+          begin try
+            let pagenumber = Hashtbl.find refnumstable i in
+              Hashtbl.find mattable pagenumber
+          with
+            _ ->
+              Printf.eprintf "page not found for bookmark or annotation dest:%s\n" (Pdfdest.string_of_destination dest);
+              Pdftransform.i_matrix
+          end
+      | _ -> Pdftransform.i_matrix
+      end
+  | _ -> Pdftransform.i_matrix
+
+(* For each bookmark, find the page its target is on, look up the appropriate matrix, and transform it. Works only for destinations. /GoTo actions are rewritten globally, separately. *)
+let change_pages_process_bookmarks mattable refnumstable pdf =
+  (*List.iter (fun (p, m) -> Printf.printf "chppb: %i = %s\n" p (Pdftransform.string_of_matrix m)) matpairs;*)
+  let bookmarks =
+    map
+      (fun m ->
+         let tr = change_pages_find_matrix m.Pdfmarks.target mattable refnumstable in
+           if tr <> Pdftransform.i_matrix then Pdfmarks.transform_bookmark tr m else m)
+      (Pdfmarks.read_bookmarks pdf)
+  in
+    Pdfmarks.add_bookmarks bookmarks pdf 
+
+let rewrite_dest pdf mattable refnumstable dest =
+  let parsed_dest = Pdfdest.read_destination pdf dest in
+  let tr = change_pages_find_matrix parsed_dest mattable refnumstable in
+    (*Printf.printf "tr is %s\n" (Pdftransform.string_of_matrix tr);*)
+    if tr <> Pdftransform.i_matrix then
+      let transformed = Pdfdest.transform_destination tr parsed_dest in
+      let new_dest = Pdfdest.pdfobject_of_destination transformed in
+        Some (Pdf.addobj pdf new_dest)
+    else
+      None
+
+let rewrite_action pdf mattable refnumstable action =
+  begin match Pdf.lookup_direct pdf "/S" action with
+  | Some (Pdf.Name "/GoTo") ->
+    begin match Pdf.lookup_direct pdf "/D" action with
+    | Some dest ->
+        begin match rewrite_dest pdf mattable refnumstable dest with
+        | Some objnum ->
+            Some (Pdf.add_dict_entry action "/D" (Pdf.Indirect objnum))
+        | None -> None
+        end
+    | _ -> None
+    end
+  | _ -> None
+  end
+
+(* For each page, find its annotations. For each, transform its annotations *)
+let change_pages_process_annotations mattable refnumstable pdf =
+   iter2
+     (fun page pnum ->
+        (*Printf.eprintf "Page %i...\n" pnum;*)
+        match Pdf.lookup_direct pdf "/Annots" page.rest with
+        | Some (Pdf.Array annots) ->
+            iter
+              (fun annotobj ->
+                match annotobj with Pdf.Indirect i ->
+                  let annot = Pdf.lookup_obj pdf i in
+                  (* Find its destination, if it has one. Either in /Dest or /A *)
+                  begin match Pdf.lookup_direct pdf "/Subtype" annot with
+                  | Some (Pdf.Name "/Link") ->
+                      begin match Pdf.lookup_direct pdf "/Dest" annot with
+                      | Some dest ->
+                          begin match rewrite_dest pdf mattable refnumstable dest with
+                          | Some objnum ->
+                              let new_annot = Pdf.add_dict_entry annot "/Dest" (Pdf.Indirect objnum) in
+                                Pdf.addobj_given_num pdf (i, new_annot)
+                          | None -> ()
+                          end
+                      | _ ->
+                          begin match Pdf.lookup_direct pdf "/A" annot with
+                          | Some (Pdf.Dictionary _ as action) ->
+                              begin match rewrite_action pdf mattable refnumstable action with
+                              | Some action ->
+                                  let new_annot = Pdf.add_dict_entry annot "/A" action in
+                                    Pdf.addobj_given_num pdf (i, new_annot)
+                              | _ -> ()
+                              end
+                          | _ -> () 
+                          end
+                      end
+                  | _ -> ()
+                  end
+                | _ -> Printf.eprintf "change_pages_process_annotations: annotation direct\n")
+                annots
+        | None -> ()
+        | _ ->
+            Printf.eprintf "change_pages_process_annotations: /Annots not an array\n")
+     (pages_of_pagetree pdf)
+     (indx (pages_of_pagetree pdf))
+
+(* Process the /OpenAction if its destination or action points to a page which has been scaled. *)
+(* Trailer --/Root--> Catalog dict --/OpenAction--> (array = dest || dict == action). No name option. *)
+let rewrite_openaction pdf action =
+  let catalog = Pdf.catalog_of_pdf pdf in
+  let catalog = Pdf.add_dict_entry catalog "/OpenAction" action in
+    Pdf.addobj_given_num pdf (pdf.Pdf.root, catalog)
+
+let change_pages_process_openaction mattable refnumstable pdf =
+  match Pdf.lookup_direct pdf "/OpenAction" (Pdf.catalog_of_pdf pdf) with
+  | Some (Pdf.Array dest) ->
+      begin match rewrite_dest pdf mattable refnumstable (Pdf.Array dest) with
+      | Some new_dest_objnum -> rewrite_openaction pdf (Pdf.Indirect new_dest_objnum)
+      | None -> ()
+      end
+  | Some (Pdf.Dictionary action) ->
+      begin match rewrite_action pdf mattable refnumstable (Pdf.Dictionary action) with
+      | Some action -> rewrite_openaction pdf action
+      | _ -> ()
+      end
+  | _ -> ()
+
+let change_pages ?matrices ?changes change_references basepdf pages' =
   let pdf = Pdf.empty () in
     Pdf.objiter (fun k v -> ignore (Pdf.addobj_given_num pdf (k, v))) basepdf;
     let old_page_numbers = Pdf.page_reference_numbers basepdf in
@@ -626,13 +772,13 @@ let change_pages ?changes change_references basepdf pages' =
                          combine old_page_numbers new_page_numbers
                        else
                          begin
-                           Printf.printf "change_pages: No change supplied, and lengths differ\n";
+                           Printf.eprintf "change_pages: No change supplied, and lengths differ\n";
                            []
                          end
                    | Some cs ->
                        (* Turn the 1-based serial numbers into page reference numbers *)
                        try
-                         List.map
+                         map
                            (fun (x, y) ->
                             List.nth old_page_numbers (x - 1), List.nth new_page_numbers (y - 1))
                          cs
@@ -642,30 +788,33 @@ let change_pages ?changes change_references basepdf pages' =
                   Pdf.objselfmap
                     (Pdf.renumber_object_parsed pdf (hashtable_of_dictionary changes))
                     pdf;
-                  pdf
+                  match matrices with
+                    None -> pdf
+                  | Some matpairs ->
+                      let refnums = Pdf.page_reference_numbers pdf in
+                      let mattable = hashtable_of_dictionary matpairs in
+                      let refnumstable = hashtable_of_dictionary (combine refnums (indx refnums)) in
+                      let pdf =
+                        if length old_page_numbers = length new_page_numbers then
+                          change_pages_process_bookmarks mattable refnumstable pdf
+                        else
+                          begin
+                            Printf.eprintf "Pdfpage.change_pages: non-null matrices when lengths differ";
+                            pdf
+                          end
+                      in
+                        begin try change_pages_process_annotations mattable refnumstable pdf with
+                          e -> Printf.eprintf "failure in change_pages_process_annotations: %s" (Printexc.to_string e)
+                        end;
+                        begin try change_pages_process_openaction mattable refnumstable pdf with
+                          e -> Printf.eprintf "failure in change_pages_process_openaction: %s" (Printexc.to_string e)
+                        end;
+                        pdf
 
 (* Return a pdf with a subset of pages, but nothing else changed - exactly the
 same page object numbers, so bookmarks etc still work. Also sorts out bookmarks
 so only those in the range are kept. *)
 
-(* Make sure to supply refnums to speed it up, if you already have them from a
- * previous call to Pdf.page_reference_numbers *)
-let pagenumber_of_target ?fastrefnums pdf = function
- | Pdfdest.NullDestination -> 0
- | Pdfdest.NamedDestinationElsewhere _ -> 0
- | Pdfdest.Action _ -> 0
- | Pdfdest.XYZ (t, _, _, _) | Pdfdest.Fit t | Pdfdest.FitH (t, _) | Pdfdest.FitV (t, _)
- | Pdfdest.FitR (t, _, _, _, _) | Pdfdest.FitB t | Pdfdest.FitBH (t, _) | Pdfdest.FitBV (t, _) ->
-     match t with
-     | Pdfdest.OtherDocPageNumber i -> i + 1 (* If it's really a Pdfdest.OtherDocPageNumber, you must process this yourself before. *)
-     | Pdfdest.PageObject i ->
-         match fastrefnums with
-         | Some table ->
-             begin try Hashtbl.find table i with Not_found -> 0 end 
-         | None ->
-             match position_1 i (Pdf.page_reference_numbers pdf) with
-             | Some n -> n
-             | None -> 0
 
 (* Find a page indirect from the page tree of a document, given a page number. *)
 (* FIXME speed up by caching *)
@@ -817,7 +966,7 @@ let fixup_parents pdf =
 
 let pdf_of_pages ?(retain_numbering = false) basepdf range =
   let page_labels =
-    if List.length (Pdfpagelabels.read basepdf) = 0 then [] else
+    if length (Pdfpagelabels.read basepdf) = 0 then [] else
       if retain_numbering
         then Pdfpagelabels.merge_pagelabels [basepdf] [range]
         else []
@@ -988,11 +1137,11 @@ let names_used pdf =
               Some (Pdf.Name ("/Page" | "/Pages")) ->
                 begin match Pdf.lookup_direct pdf "/Resources" obj with
                   Some resources ->
-                    List.iter
+                    iter
                       (fun key ->
                          match Pdf.lookup_direct pdf key resources with
                            Some (Pdf.Dictionary d) ->
-                             List.iter
+                             iter
                                (fun (k, _) -> names := unslash k::!names)
                                d
                          | _ -> ())
@@ -1120,7 +1269,7 @@ let add_prefix pdf prefix =
                        (if resources = None then Pdf.Dictionary [] else unopt resources)
                        (Pdf.Indirect i)
                  | Some (Pdf.Array a) ->
-                     List.iter
+                     iter
                        (fix_stream
                          (if resources = None then Pdf.Dictionary [] else unopt resources))
                        a
